@@ -12,7 +12,8 @@ from sphinx.util.nodes import nested_parse_with_titles
 
 from docutils import nodes
 from docutils.nodes import Element, Node, TextElement  # noqa
-from docutils.statemachine import StringList
+from docutils.statemachine import StringList, UnexpectedIndentationError
+from docutils.parsers.rst.states import Text
 
 try:
     from sphinxcontrib import phpdomain as php  # type: ignore
@@ -396,6 +397,49 @@ def get_definition_without_template_args(data_object):
     return definition
 
 
+class InlineText(Text):
+    """
+    Add a custom docutils class to allow parsing inline text. This is to be
+    used inside a @verbatim/@endverbatim block but only the first line is
+    consumed and a inline element is generated as the parent, instead of the
+    paragraph used by Text.
+    """
+    patterns = {'inlinetext': r''}
+    initial_transitions = [('inlinetext', )]
+
+    def indent(self, match, context, next_state):
+        """
+        Avoid Text's indent from detecting space prefixed text and
+        doing "funny" stuff; always rely on inlinetext for parsing.
+        """
+        return self.inlinetext(match, context, next_state)
+
+    def eof(self, context):
+        """
+        Text.eof() inserts a paragraph, so override it to skip adding elements.
+        """
+        return []
+
+    def inlinetext(self, match, context, next_state):
+        """
+        Called by the StateMachine when an inline element is found (which is
+        any text when this class is added as the single transition.
+        """
+        startline = self.state_machine.abs_line_number() - 1
+        msg = None
+        try:
+            block = self.state_machine.get_text_block()
+        except UnexpectedIndentationError as err:
+            block, src, srcline = err.args
+            msg = self.reporter.error('Unexpected indentation.',
+                                      source=src, line=srcline)
+        lines = context + list(block)
+        text, _ = self.inline_text(lines[0], startline)
+        self.parent += text
+        self.parent += msg
+        return [], next_state, []
+
+
 class SphinxRenderer:
     """
     Doxygen node visitor that converts input into Sphinx/RST representation.
@@ -431,7 +475,44 @@ class SphinxRenderer:
         if self.context.domain == '':
             self.context.domain = self.get_domain()
 
+    # XXX: fix broken links in XML generated by Doxygen when Doxygen's
+    # SEPARATE_MEMBER_PAGES is set to YES; this function should be harmless
+    # when SEPARATE_MEMBER_PAGES is NO!
+    #
+    # The issue was discussed here: https://github.com/doxygen/doxygen/pull/7971
+    #
+    # A Doxygen anchor consists of a 32-byte string version of the results of
+    # passing in the stringified identifier or prototype that is being "hashed".
+    # An "a" character is then prefixed to mark it as an anchor. Depending on how
+    # the identifier is linked, it can also get a "g" prefix to mean it is part
+    # of a Doxygen group. This results in an id having either 33 or 34 bytes
+    # (containing a "g" or not). Some identifiers, eg enumerators, get twice that
+    # length to have both a unique enum + unique enumerator, and sometimes they
+    # get two "g" characters as prefix instead of one.
+    def _fixup_separate_member_pages(self, refid: str) -> str:
+        if refid:
+            parts = refid.rsplit("_", 1)
+            if len(parts) == 2 and parts[1].startswith("1"):
+                anchorid = parts[1][1:]
+                if len(anchorid) in set([33, 34]) and parts[0].endswith(anchorid):
+                    return parts[0][:-len(anchorid)] + parts[1]
+                elif len(anchorid) > 34:
+                    index = 0
+                    if anchorid.startswith('gg'):
+                        index = 1
+                        _len = 35
+                    elif anchorid.startswith('g'):
+                        _len = 34
+                    else:
+                        _len = 33
+                    if parts[0].endswith(anchorid[index:_len]):
+                        return parts[0][:-(_len - index)] + parts[1]
+
+        return refid
+
     def get_refid(self, refid: str) -> str:
+        if self.app.config.breathe_separate_member_pages:  # type: ignore
+            refid = self._fixup_separate_member_pages(refid)
         if self.app.config.breathe_use_project_refids:  # type: ignore
             return "%s%s" % (self.project_info.name(), refid)
         else:
@@ -1307,6 +1388,25 @@ class SphinxRenderer:
     def visit_highlight(self, node) -> List[Node]:
         return self.render_iterable(node.content_)
 
+    def _nested_inline_parse_with_titles(self, content, node) -> str:
+        """
+        This code is basically a customized nested_parse_with_titles from
+        docutils, using the InlineText class on the statemachine.
+        """
+        surrounding_title_styles = self.state.memo.title_styles
+        surrounding_section_level = self.state.memo.section_level
+        self.state.memo.title_styles = []
+        self.state.memo.section_level = 0
+        try:
+            return self.state.nested_parse(content, 0, node, match_titles=1,
+                                           state_machine_kwargs={
+                                               'state_classes': (InlineText,),
+                                               'initial_state': 'InlineText'
+                                           })
+        finally:
+            self.state.memo.title_styles = surrounding_title_styles
+            self.state.memo.section_level = surrounding_section_level
+
     def visit_verbatim(self, node) -> List[Node]:
         if not node.text.strip().startswith("embed:rst"):
             # Remove trailing new lines. Purely subjective call from viewing results
@@ -1315,6 +1415,8 @@ class SphinxRenderer:
             # Handle has a preformatted text
             return [nodes.literal_block(text, text)]
 
+        is_inline = False
+
         # do we need to strip leading asterisks?
         # NOTE: We could choose to guess this based on every line starting with '*'.
         #   However This would have a side-effect for any users who have an rst-block
@@ -1333,11 +1435,19 @@ class SphinxRenderer:
             lines = map(lambda text: text.replace("///", "   ", 1), lines)
             node.text = "\n".join(lines)
 
-        # Remove the first line which is "embed:rst[:leading-asterisk]"
-        text = "\n".join(node.text.split(u"\n")[1:])
+        elif node.text.strip().startswith("embed:rst:inline"):
+            # Inline all text inside the verbatim
+            node.text = "".join(node.text.splitlines())
+            is_inline = True
 
-        # Remove starting whitespace
-        text = textwrap.dedent(text)
+        if is_inline:
+            text = node.text.replace("embed:rst:inline", "", 1)
+        else:
+            # Remove the first line which is "embed:rst[:leading-asterisk]"
+            text = "\n".join(node.text.split(u"\n")[1:])
+
+            # Remove starting whitespace
+            text = textwrap.dedent(text)
 
         # Inspired by autodoc.py in Sphinx
         rst = StringList()
@@ -1345,13 +1455,19 @@ class SphinxRenderer:
             rst.append(line, "<breathe>")
 
         # Parent node for the generated node subtree
-        rst_node = nodes.paragraph()
+        if is_inline:
+            rst_node = nodes.inline()
+        else:
+            rst_node = nodes.paragraph()
         rst_node.document = self.state.document
 
         # Generate node subtree
-        nested_parse_with_titles(self.state, rst, rst_node)
-        # TODO: hmm, what is supposed to be returned? something is strange with the types
-        return rst_node  # type: ignore
+        if is_inline:
+            self._nested_inline_parse_with_titles(rst, rst_node)
+        else:
+            nested_parse_with_titles(self.state, rst, rst_node)
+
+        return [rst_node]
 
     def visit_inc(self, node) -> List[Node]:
         if node.local == u"yes":
